#include "common.h"

chessclock::chessclock(){
     movesPerRound=1;
     timePerRound=1000;
     margin=200;
     timeLeft[side(BLACK)]=timePerRound;
     timeLeft[WHITE]=timePerRound;
     movesLeft[side(BLACK)]=movesPerRound;
     movesLeft[WHITE]=movesPerRound;

     runningSide=NOPLAYER;
	 incrementalClock=TRUE;
	 movesHalfGame=50;
}

//set time for each side. Stops the clock if running.
void chessclock::setClock(int ms, int moves){
     stopClock();
     movesPerRound=moves;
     timePerRound=ms;
     timeLeft[side(BLACK)]=ms-margin;
     timeLeft[WHITE]=ms-margin;
     movesLeft[side(BLACK)]=moves;
     movesLeft[WHITE]=moves;
	 if (moves==0){
		 incrementalClock=FALSE;
		 if(ms<=120000){
			 movesHalfGame=25;
		 }
		 else if (ms<=480000){
			 movesHalfGame=35;
		 }
		 else{
			 movesHalfGame=50;
		 }
	 }
	 else{incrementalClock=TRUE;}
}

//increase the time of one side by ms
void chessclock::increaseTime(int playingSide, int ms){
     timeLeft[side(playingSide)]+=ms;     
}

//set the time of one side by ms
void chessclock::setTime(int playingSide, int ms){
     timeLeft[side(playingSide)]=ms-margin;     
}


//starts clock of one side, stops clock for the other      
void chessclock::startClock(int startClockSide){
     //If clock was already running at the opposite side then substract the
     //time that has past by.
     if ( (runningSide==WHITE && startClockSide==BLACK)|| 
          (runningSide==BLACK && startClockSide==WHITE) ){
        stopTime=timer1.get_ms();
        timeLeft[side(runningSide)]-= (stopTime-startTime);
        //and also decrease numbers of moves by one
        movesLeft[side(runningSide)]--;
        //No moves left to do in this time? Add moves and time
        if (movesLeft[side(runningSide)]==0 && incrementalClock){
           movesLeft[side(runningSide)]=movesPerRound;
           timeLeft[side(runningSide)]+=timePerRound;   
        }
     }
     if (runningSide!=startClockSide){
         //set running side
         runningSide=startClockSide;
         //and start the timer for the next round.
         startTime=timer1.get_ms();
     }
}

//stops clock for both sides
void chessclock::stopClock(){
     if (runningSide==WHITE || runningSide==BLACK){
        stopTime=timer1.get_ms();
        timeLeft[side(runningSide)]-= (stopTime-startTime);
     }
     runningSide=NOPLAYER;
}

//returns ms left for white/black     
int chessclock::getTimeLeft(int playingSide){
     if (runningSide==WHITE || runningSide==BLACK){
        stopTime=timer1.get_ms();
        timeLeft[side(runningSide)]-= (stopTime-startTime);
        startTime=timer1.get_ms();
     }
    return timeLeft[side(playingSide)];
}


//returns ms left for next move of white/black     
int chessclock::getTimeNextMove(int playingSide){
     if (runningSide==WHITE || runningSide==BLACK){
        stopTime=timer1.get_ms();
        timeLeft[side(runningSide)]-= (stopTime-startTime);
        startTime=stopTime;
     }
	 if(incrementalClock){
		int timeLeftNM=timeLeft[side(playingSide)]/movesLeft[side(playingSide)];
		if (timeLeftNM<(2 * timePerRound/movesPerRound) ){
			return timeLeftNM;
		} 
		else{
			return 2 * timePerRound/movesPerRound;
		}
	 }
	 else{
	 //we are using a fixed time for the whole game
		 if (timeLeft[side(playingSide)]*2>timePerRound){
			//we havenot consumed half of the available time
			return timePerRound/(movesHalfGame*2);
		 }
		 else{
			 if(timeLeft[side(playingSide)]==0){
				 return 0;
			 }
			 else{
				 //we suppose we need another fixed number of moves to end the game
				 return timeLeft[side(playingSide)]/movesHalfGame;
			 }
		 }

	 }
}
